' Lawson Labs, Inc.
' 3239 Phoenixville Pike, Malvern, PA 19355
' 800 321-5355 or 610 725-8800
' lawsnlab@netaxs.com
' Model 203 24-Bit Delta-Sigma Converter            Rev A
' October 1, 1997                                   microcode version 203

'***************************************************************************
'****  Define variables                                                 ****
'***************************************************************************
SETVARS:

  'constants
  FALSE = 0
  TRUE = -1

  'these three flags can be set by the user to enable features
  ' that the menus do not give direct access to
  CHKSUMFLG% = TRUE             'Set for checksum scan mode
  TRGRSCANFLG% = FALSE          'Set for triggered scan
  EDGE% = 1                     '1 = positive edge trig, 0 = negative edge

  'other flags
  SIGNONFLAG% = TRUE            'set here, and for average or rate change
  ECHOFLAG% = FALSE             'TRUE for echo test during signon
  ChanChgFlag% = TRUE           'Set when channel has been changed, cleared
				'when running average array has refilled
  POLLEDFLAG% = TRUE            'TRUE except during scanning
  CANCELFLAG% = FALSE           'Set when a keystoke stops an A/D conversion
  DIGINFLAG% = FALSE            'Set when digital input is requested during
				'scanning. Cleared when digital data received
				
  PICCRYSTAL# = 19.6608         'microcontroller crystal frequency in Mhz
  Vref = 5                      'normal reference voltage is 5. Other
				'voltages require custom hardware
  Range = 2                     '2 for standard +/-5v board (1.2 for -1 to 5v)
  MAXCNT% = &H3FFF              'Base period for analog outputs
				'&H3FFF is maximum value for best resolution
				'minimum is &H2FF for 10 bits. Intermediate
				'values are fine, but they must end in &HFF
  SDGROUP = 20                  'group size for standard deviation
  REDIM SD#(SDGROUP)            'standard deviation array
  SDTOT# = 0                    'total for standard deviation calc
  SDCOUNT% = 1                  'counter for standard deviation group
  REDIM BAUD$(5)                'Baud rate strings
  BAUD$(0) = "9600"
  BAUD$(1) = "4800"
  BAUD$(2) = "2400"
  BAUD$(3) = "1200"
  BAUD$(4) = "600"
  BAUD$(5) = "300"
  LASTDIGOUT% = 256                  'initalize to invalid code
  LASTDIGINP% = 256                  'initalize to invalid code
  DIGINP$(0) = "Active (low)   "     'index with LASTDIGINP%
  DIGINP$(1) = "Inactive (high)"     '  "    "       "
  GOSUB READCONFIG                   'start from last set-up saved to disk
  GOSUB CALCVARS                     'do calculations based on configuration

'***************************************************************************
'****  COMMUNICATIONS MENU                                              ****
'****  provides for serial port set-up and test                         ****
'***************************************************************************

COMMENU:
  KEY OFF: CLS
  LOCATE 6, 22: PRINT "SERIAL COMMUNICATION  MENU"
  LOCATE 10, 20: PRINT "1) Baud Rate              "; BAUD$(BAUD%)
  LOCATE 12, 20: PRINT "2) Serial port            "; COM$;
  LOCATE 14, 20: PRINT "3) Input buffer          "; INPBUF%; " bytes"
  LOCATE 16, 20: PRINT "4) Serial check           "
  LOCATE 18, 16: PRINT "Enter) Start";
  LOCATE 20, 18: PRINT "Esc) Exit program";
  A$ = INPUT$(1)
  IF A$ = "1" THEN BAUD% = (BAUD% + 1) MOD 6: SIGNONFLAG% = TRUE
  IF A$ = "2" THEN IF COM$ = "COM1:" THEN COM$ = "COM2:" ELSE COM$ = "COM1:"
  IF A$ = "3" THEN
    LOCATE 25, 1: INPUT "            Buffer size (bytes) 1-32767"; TEMP
    IF (TEMP >= 1) AND (TEMP <= 32767) THEN INPBUF% = TEMP
  END IF
  IF A$ = "4" THEN
    ECHOFLAG% = TRUE: GOSUB SIGNON    'test communications
    GOSUB SENDMRESET: ECHOFLAG% = FALSE
  END IF
  IF A$ = CHR$(13) THEN GOSUB POLLOOP: GOTO COMMENU
  IF A$ = CHR$(27) THEN GOSUB EXITPROGRAM
  GOTO COMMENU
  
'**************************************************************************
'**************************   SUBROUTINES   *******************************
'**************************************************************************
 
CALCVARS:                                'Calculate after loading config file
  PICCYCLETIME# = .000004 / PICCRYSTAL#  'microcontroller cycle time in secs
  EightBitTime# = PICCYCLETIME# * 256    'time to get a 8-bit subanswer
			     'SUBSAMPLE% is the number of subanswers to total
					 '  must be between 128 and 255
  IF SUBSAMPLE% < 128 OR SUBSAMPLE% > 255 THEN SUBSAMPLE% = 160
					  '160 for 60 Hz rejection
					  'use 192 for 50 Hz rejection
  SamplePeriod# = EightBitTime# * SUBSAMPLE%
  Maxrate = 1 / SamplePeriod#             'max datarate-limited scan rate
  TimeBase% = 256 - SUBSAMPLE%            'TimeBase% is means for passing
					  '  SUBSAMPLE% to Model 203
  REDIM A#(RunningAvg%)                   'array for running average
  GOSUB CALCWAIT                          'determine conversion time
RETURN

'***************************************************************************
'****  SIGNON: -  Establishes communications with the model 203         ****
'****  USES    -  INPBYTE%, WAKEUPCALLS%, BAUD%, SETBAUD, MODEREGLO%    ****
'****             MODEREGMID%, PLACEHOLDER%, #1, ECHOFLAG%,             ****
'****             POLLEDFLAG%, MODEOK%, PLACEHOLDER%, COM$, LOC(1)      ****
'****  CHANGES -  OUTBYTE%, OUTB1%, CHECKSUM%, NUMCHARS%, WAITIME, XX   ****
'****  CALLS   -  GETCHAR, SETBAUD, ECHOTEST, SENDPACKET                ****
'****             WAITXX, PRINTERROR, EMPTYBUFFER                       ****
'***************************************************************************


'The operating mode and rate are set by sending a 24-bit word to the M203.
'The mode word is sent as three 8-bit bytes.
'The large number of "spare" or unused bytes are mostly needed to maintain
'compatbility with the Model 202.

' MODEREGHI%    not used

' MODEREGMID%  ' bit 7   24bit/16bit         WL%       (wordlength)
		   ' 6-0   not used       The Model 203 can operate in 16-bit
		   'mode for compatibility with the Model 202.
		   'only 24-bit mode is supported in this driver.
' MODEREGLO%   = TimeBase% periods to totalize for each result
' TimeBase%    = Timebase for A/D conversion. Calculated from SUBSAMPLE%

SIGNON:
  WAKEUPCALLS% = 0                      'counter for wakeup bytes sent
  e$ = "": ON ERROR GOTO 1000
WAKEMODE:
  GOSUB SETBAUD
  INPBYTE% = 0
  OUTBYTE% = &H0                        'command to reset the M203
  CLS : LOCATE 10, 28: PRINT "Signing on. Try #";
WAKEUP:
  WAKEUPCALLS% = WAKEUPCALLS% + 1       '25 tries
  LOCATE 10, 45: PRINT WAKEUPCALLS%;
  IF WAKEUPCALLS% = 25 THEN
    IF e$ = "" THEN e$ = " can't sign on"
    GOTO PRINTERROR
  END IF
  XX = .2: GOSUB WAITXX
  PRINT #1, CHR$(OUTBYTE%);
  XX = .2: GOSUB WAITXX
  GOSUB EMPTYBUFFER                     'sets INPBYTE% to last byte in buffer
  IF INPBYTE% <> 3 THEN GOTO WAKEUP     '203 sends 3 if ready to sign on
					'it may take several tries to succeed
  XX = .2: GOSUB WAITXX                 '203 is now waiting for sign-on token
  Checksum% = 0                         'clear running checksum
  PRINT #1, CHR$(&H88);                 'send signon token
  XX = .1: GOSUB WAITXX                 'wait .1 second
  PRINT #1, CHR$(BAUD%);                'send baud code
  WAITIME = 1: NUMCHARS% = 1: GOSUB WAITCHAR
  GOSUB GETCHAR
  IF INPBYTE% <> BAUD% THEN e$ = " BAUD echo wrong": GOTO WAKEUP
  CLS
  XX = .1: GOSUB WAITXX                 'wait .1 sec for stabilization
  IF ECHOFLAG% THEN GOSUB ECHOTEST      'do a communication check
  OUTBYTE% = &H0                        'echo-off token
  PRINT #1, CHR$(OUTBYTE%);             'send echo-off token
  MODEREGMID% = WL%                     'send word length
  OUTB1% = MODEREGHI%: OUTB2% = MODEREGMID%
  GOSUB SENDPACKET                      'send timing data
  OUTB1% = MODEREGLO%: OUTB2% = TimeBase%
  GOSUB SENDPACKET                      'send TimeBase%
  OUTB1% = Div%: OUTB2% = PLACEHOLDER%
  GOSUB SENDPACKET                      'for backward compatibility


' move comment below down one line for polled-only sign on
' OUTB2% = 1   'does not send scan setup data to M203
  OUTB2% = 0   'send scan setup data to M203

 
  GOSUB SENDPACKET
  GOSUB READMODE                        'Model 203 sends back timing data
					'bytes to confirm their receipt
  ON ERROR GOTO 0                       'turn off error trapping

  IF OUTB2% = 1 THEN GOTO SkipSendScanData
   
  'Only first packet used, SCAN2% is always 0. Others are for compatibility.
 
  OUTB1% = SCAN%: OUTB2% = SCAN1%: GOSUB SENDPACKET
  OUTB1% = SCAN2%: OUTB2% = PLACEHOLDER%: GOSUB SENDPACKET
  OUTB1% = PLACEHOLDER%: OUTB2% = PLACEHOLDER%: GOSUB SENDPACKET
  OUTB1% = PLACEHOLDER%: OUTB2% = PLACEHOLDER%: GOSUB SENDPACKET
  OUTB1% = PLACEHOLDER%: OUTB2% = 0: GOSUB SENDPACKET

SkipSendScanData:
  XX = .2: GOSUB WAITXX
  'sign-on should now be complete. Always calibrate before taking data.
  GOSUB CALCWAIT
  SIGNONFLAG% = FALSE
RETURN

READMODE:
  NUMCHARS% = 3: MODEOK% = TRUE
  GOSUB WAITCHAR                        'wait for 3 characters
  GOSUB GETCHAR
  IF (INPBYTE% AND &H1F) <> (MODEREGHI% AND &H1F) THEN MODEOK% = FALSE
  GOSUB GETCHAR
  IF INPBYTE% <> MODEREGMID% THEN MODEOK% = FALSE
  GOSUB GETCHAR
  IF INPBYTE% <> MODEREGLO% THEN MODEOK% = FALSE
  IF MODEOK% = FALSE THEN
    BEEP: e$ = " Wrong mode registers"    'beep if mode not properly received
    RETURN WAKEMODE
  END IF
RETURN

'***************************************************************************
'****  Error handler for signon                                         ****
'***************************************************************************

1000 CLOSE #1
  GOSUB SETBAUD
  RESUME NEXT     'ignore serial errors during sign-on

EMPTYBUFFER:
  IF LOC(1) <> 0 THEN GOSUB GETCHAR: GOTO EMPTYBUFFER
RETURN


'***************************************************************************
'****  Open selected serial port at selected BAUD and buffer size       ****
'****  USES    -  COM$, BAUD%, INPBUF%                                  ****
'****  CALLS   -                                                        ****
'***************************************************************************
SETBAUD:                        'set serial port to selected baud rate
  CLOSE
  S$ = COM$ + BAUD$(BAUD%) + ",N,8,1,BIN,RS,CS,DS,CD,RB" + STR$(INPBUF%) + ",TB2"
  OPEN S$ FOR RANDOM AS #1
RETURN

'***************************************************************************
'****  Gathers data in polled mode                                      ****
'****  USES    -  GAIN%, CANCELFLAG%, CHANNEL%                          ****
'****  CHANGES -  POLLEDFLAG%, MAX, MIN                                 ****
'****  CALLS   -  SIGNON, SETCCODE, WAITXX, SYSCAL                      ****
'****             GetSubConversion, VOLTSCALE, SD, DOCOMMAND            ****
'****             SCANMENU, DOMRESET, VOLTDISPLAY                       ****
'***************************************************************************
POLLOOP:
  POLLEDFLAG% = TRUE                            'Flag mode of operation
  GOSUB SIGNON
  GOSUB SYSCAL
  GOSUB GetVersion
  GOSUB SETCCODE                                'select input channel
  CLS
  h$ = "    COUNT       VOLTS       MAX-MIN    RESOLUTION    CHANNEL     EF BITS of"
  LASTCSR = 3
  GOSUB INITDACS                                'initialize analog outputs
  A$ = " "
POLLOOP2:
  IF A$ <> "" THEN                              're-label screen if needed
    LOCATE 1, 1: PRINT h$;                      'print header
    PRINT USING "###.#"; Bits;
    LOCATE 25, 1
    PRINT " SPACE=MAX-MIN, M=Menu, N=scaN, Esc=serial menu";
    LOCATE 2, 1: PRINT SPACE$(80): LOCATE 24, 1: PRINT SPACE$(80);
    A$ = ""                                      'A$ holds last command
  END IF
  LOCATE LASTCSR, 1
  GOSUB GetConv                                 'get an A/D conversion
  IF CANCELFLAG% THEN GOTO KEYSTROKE            'handle keystroke first
  PRINT USING " ######## "; Count&;             'print raw count
  GOSUB VOLTDISPLAY                             'display voltage
  IF Max < volts THEN Max = volts               'update maximum
  IF Min > volts THEN Min = volts               'update minimum
  PRINT USING "   ###.######"; Max - Min;       'display the difference
  GOSUB SD                                      'calculate standard deviation
  PRINT USING "    ###.###### "; STDV;
  PRINT "    Chan="; Channel%;                  'display current channel
  RMSNOISE = STDV / GAIN#                       'RMSNOISE=RMS noise in counts
  IF RMSNOISE < 1 THEN RMSNOISE = 1             'avoid negative log result
  NOISEBITS = LOG(RMSNOISE) * 1.442695          'RMS noise in bits
  EFFECTIVE = Bits - NOISEBITS                  'bits = maximum resolution
  PRINT USING "   ###.#    "; EFFECTIVE         'total res. - noise in bits
  IF CSRLIN = 24 THEN LOCATE 3, 1
  LASTCSR = CSRLIN

KEYSTROKE:                               'process keyboard input, if any
  IF A$ = "" THEN A$ = UCASE$(INKEY$)    'if no keystroke, check for one
  IF A$ <> "" THEN GOSUB DOCOMMAND
  IF A$ = "M" THEN GOSUB COMMANDMENU
  IF A$ = "N" THEN SIGNONFLAG% = TRUE: GOSUB SCANMENU: CLS : GOTO POLLOOP2
  IF A$ = CHR$(27) THEN GOSUB DOMRESET: RETURN
  GOTO POLLOOP2

'***************************************************************************
'****   Perform system calibration                                      ****
'****   Resets orginal channel when done                                ****
'***************************************************************************

SYSCAL:
  LOCATE 25, 1
  PRINT "                    System Calibration                             ";
  ORGCHAN% = Channel%
SYSCAL1:                     'use this entry from scanning code
  Channel% = 7: GOSUB SETCCODE
  XX = .2: GOSUB WAITXX
  GOSUB GetFullConv                       'do offset cal on channel 7
  IF CANCELFLAG% THEN GOTO CALSTOPPED
  GOSUB OffsetCal: Channel% = 6: GOSUB SETCCODE
  XX = .2: GOSUB WAITXX
  GOSUB GetFullConv                       'do full scale cal on channel 6
  IF CANCELFLAG% THEN GOTO CALSTOPPED
  GOSUB FullScaleCal
EXITSYSCAL:
  Channel% = ORGCHAN%: GOSUB SETCCODE      'reselect orginal channel
  A$ = " ": LOCATE 25, 1: PRINT SPACE$(79);
RETURN

CALSTOPPED:
  LOCATE 25, 1
  PRINT "NOT Calibrated. Press a key to continue or S for System Calibration";
  B$ = INPUT$(1)
  IF UCASE$(B$) = "S" THEN Channel% = ORGCHAN%: GOTO SYSCAL
  GOTO EXITSYSCAL

OffsetCalMessage:
  LOCATE 25, 1
  PRINT "                  Offset Calibration                             ";
OffsetCal:
  IF ChanChgFlag% THEN
    GOSUB GetFullConv
    IF CANCELFLAG% THEN BEEP       'system not calibrated
  END IF
       OFFSET& = Total&
  A$ = " ": LOCATE 25, 1: PRINT SPACE$(79);
RETURN
 
FSCalMessage:
  LOCATE 25, 1
  PRINT "             Full Scale Calibration                               ";
FullScaleCal:
  IF ChanChgFlag% THEN
    GOSUB GetFullConv
    IF CANCELFLAG% THEN BEEP
  END IF
  CNT& = Total& - OFFSET&
  IF ABS(CNT&) <= 1000 THEN
    LOCATE 25, 1
    PRINT " Too small a range for calibration. Press any key to continue ";
    B$ = INPUT$(1): LOCATE 25, 1: PRINT SPACE$(79);
  ELSE
    GAIN# = Vref / (CNT&)
    Bits = LOG(ABS(Range * CNT&)) * 1.442695     'number of bits for 10V range
  END IF                                 'used for effective resolution calc
  A$ = " ": LOCATE 25, 1: PRINT SPACE$(79);
RETURN

'***************************************************************************
'****  Get a conversion, works for polled & scanning modes              ****
'****  If new channel, take LOCALAVG points to fill RunningAvg array    ****
'****  else get one new data point                                      ****
'****  USES    - WORDCNT%, WL%, CANCELFLAG%                             ****
'****  CHANGES - COUNT&, COUNT1#, COUNT2#, CHECKSUM%                    ****
'****  CALLS   - SENDPACKET, WAITCHARORKEY, GETCHAR                     ****
'***************************************************************************
GetFullConv:           'do extra conversion and discard result if new channel
  IF ChanChgFlag% THEN Total& = 0: APTR% = 0: GOSUB GetSubConversion
AV1:                    
  GOSUB GetSubConversion
  IF ChanChgFlag% = FALSE THEN Total& = Total& - A#(APTR%)  'subtract oldest
  A#(APTR%) = Count&                                'add current count
  Total& = Total& + Count&
  IF APTR% = RunningAvg% - 1 THEN ChanChgFlag% = FALSE   'clear flag after
  APTR% = (APTR% + 1) MOD RunningAvg%                    'array is filled
  IF ChanChgFlag% THEN GOTO AV1
AV2:
  volts = (Total& - OFFSET&) * GAIN#
EXITGETCONV:
RETURN

' this entry point speeds settling by preloading the running average
' array with a single current data point after a channel change
' if not a new channel, it behaves the same as GetFullConv above
GetConv:            
  IF ChanChgFlag% THEN
    IF POLLEDFLAG% THEN GOSUB GetSubConversion   'discard first if polled mode
    GOSUB GetSubConversion
    FOR APTR% = 0 TO RunningAvg% - 1: A#(APTR%) = Count&: NEXT
    Total& = Count& * RunningAvg%: APTR% = 0: ChanChgFlag% = FALSE
    GOTO AV2
  END IF
  GOTO AV1

' Routine to get one analog input result from the Model 203
' If polled mode, request it first.
GetSubConversion:
  IF POLLEDFLAG% THEN
    OUTB1% = &H81                               'get conversion token
    OUTB2% = 0                                  'argument ignored
    GOSUB SENDPACKET
    NUMCHARS% = WORDCNT% + 1: GOSUB WAITCHARORKEY   'wait for characters
    IF CANCELFLAG% THEN RETURN EXITGETCONV
    GOSUB GETCHAR
    IF INPBYTE% <> &H81 THEN e$ = " Get conv token failure": GOTO PRINTERROR
  ELSE
    NUMCHARS% = WORDCNT%: GOSUB WAITCHAR   'wait for 2 or 3 char's
  END IF
NEXTCONV:
  GOSUB GETCHAR
  BYTE1& = INPBYTE%
  SCANSUM% = Checksum%   'preserve checksum in case mode is checksum scanning
  GOSUB GETCHAR: BYTE2& = INPBYTE%
  IF WL% = 128 THEN GOSUB GETCHAR: BYTE3& = INPBYTE% ELSE BYTE3& = 0

'following lines are used retrieve a digital input with checksum from
'the analog data stream while scanning.
'Note that this logic only works in 24-bit mode.
'A similar method works for digital inputs without checksum in 16-bit mode.
  IF DIGINFLAG% AND (BYTE1& = &H8E) THEN
    IF (((SCANSUM% + BYTE2&) MOD 256) = BYTE3&) AND (BYTE2& < 2) THEN
	LASTDIGINP% = BYTE2&: DIGINFLAG% = FALSE     'found digital input
	NUMCHARS% = WORDCNT%: GOSUB WAITCHAR         'go back again to get
	GOTO NEXTCONV                                'conversion result
    END IF
  END IF
 
  IF (NOT POLLEDFLAG%) AND CHKSUMFLG% THEN           'checksum scanning only
    GOSUB GETCHAR
    IF SCANSUM% <> INPBYTE% THEN                     'verify checksum
	e$ = " Checksum error": GOTO PRINTERROR
    END IF
  END IF
  Count& = BYTE1& + BYTE2& * 256 + BYTE3& * 65536    'calc raw counts
EXITGetSubConversion:
RETURN

'***************************************************************************
'****  Calculate standard deviation, STDV                               ****
'****  USES    - SD#, COUNT&, SDCOUNT%, SDGROUP                          ****
'****  CHANGES - SDTOT#, STDV, SDCOUNT%                                  ****
'***************************************************************************
SD:
  SD#(SDCOUNT%) = volts
  SDTOT# = SDTOT# + volts
  IF SDCOUNT% <> SDGROUP THEN SDCOUNT% = SDCOUNT% + 1: GOTO EXITSD
  STDV = 0
  SDTOT# = SDTOT# / SDGROUP
  FOR N = 1 TO SDGROUP: STDV = STDV + ((SDTOT# - SD#(N)) ^ 2): NEXT N
  STDV = SQR(STDV / (SDGROUP - 1))

CLEARSDCOUNT:
  SDTOT# = 0: SDCOUNT% = 1
EXITSD:
RETURN

'***************************************************************************
'****  Display volts                                                    ****
'***************************************************************************
VOLTDISPLAY:
  PRINT USING "   ##.######"; volts;            'display volts
RETURN

'***************************************************************************
'****  Clear maximum & minimum voltage                                  ****
'****  CHANGES - MAX, MIN                                               ****
'***************************************************************************
ZERODELTA:
  Max = -10: Min = 10
RETURN

COMMANDMENU:
  CLS
  LOCATE 2, 30: PRINT "COMMAND  MENU"; SPACE$(20); "version = ";
    : PRINT USING "###"; VERSION;
  LOCATE 6, 5: PRINT "C) A/D input Channel     "; Channel%
  LOCATE 8, 5: PRINT "A) Analog Output settings ";
  LOCATE 10, 5
  PRINT USING "R) sample Rate (max = ###.#"; Maxrate;
  PRINT USING ") ###.#"; Maxrate / TotalPeriods%;
  LOCATE 12, 5: PRINT "L) Local averaging "; RunningAvg%;
  LOCATE 14, 5: PRINT "S) System calibration"
  LOCATE 18, 3: PRINT "Esc) To exit";
  LOCATE 6, 45: PRINT "D) Digital input   ";
  IF A$ = "D" THEN PRINT DIGINP$(LASTDIGINP%);
  LOCATE 8, 45: PRINT "U) digital oUtput   ";
  IF LASTDIGOUT% <> 256 THEN PRINT LASTDIGOUT% 'print only after "U" command
  LOCATE 10, 45: PRINT "K) checKsum         ";
  IF A$ = "K" THEN PRINT "OK"
  LOCATE 12, 45: PRINT "O) Offset calibration";
  LOCATE 14, 45: PRINT "F) Fullscale calibration";
  LOCATE 18, 41: PRINT "Enter) Start"
  A$ = UCASE$(INPUT$(1))
  IF A$ = CHR$(27) THEN RETURN
  IF A$ = CHR$(13) THEN : CLS : RETURN
  GOSUB DOCOMMAND
  GOTO COMMANDMENU

DOCOMMAND:
  IF A$ = "A" THEN GOSUB ANALOGOUT: CLS         'set analog outputs
  IF A$ = "C" THEN GOSUB CHANNELCHG             'change channel
  IF A$ = "D" THEN GOSUB GETDIGIN               'read the digital inputs
  IF A$ = "F" THEN GOSUB FSCalMessage           'calibrate full scale
  IF A$ = "K" THEN GOSUB GETCHKSUM              'display running checksum
  IF A$ = "L" THEN GOSUB LocalAvg               'select local averaging
  IF A$ = "O" THEN GOSUB OffsetCalMessage       'calibrate offset
  IF A$ = "S" THEN GOSUB SYSCAL                 'system cal
  IF A$ = "R" THEN GOSUB SetPeriod              'determine rate
  IF A$ = "U" THEN GOSUB SETDIGOUT              'set digital outputs
  IF A$ = "*" THEN GOSUB FullScaleCalToValue    'user entered full scale cal
  IF A$ = " " THEN GOSUB ZERODELTA              'reset max-min
RETURN

'***************************************************************************
'**** Get checksum from M203 & display it. If checksum is wrong, then   ****
'**** pause & flag an error.                                            ****
'**** USES    - CHECKSUM%                                               ****
'**** CHANGES - CHECKSUM%, PRINTSUM%                                    ****
'**** CALLS   - SENDPACKET, WAITCHAR, GETCHAR, PRINTERROR, CLEARCOMMAND ****
'***************************************************************************
GETCHKSUM:
  OUTB1% = &H87                         'send checksum token
  OUTB2% = 0                            'argument ignored
  GOSUB SENDPACKET
  NUMCHARS% = 2: GOSUB WAITCHAR         'wait for 2 char
  GOSUB GETCHAR
  IF OUTB1% <> INPBYTE% THEN e$ = " Checksum token wrong ": GOTO PRINTERROR
  GOSUB GETCHAR
  IF Checksum% <> (INPBYTE% + INPBYTE%) MOD 256 THEN  'M203's checksum must
    PRINT "M201 checksum (HEX) "; HEX$(INPBYTE%),     'be added in before
    PRINT " BASIC checksum (HEX) "; HEX$(Checksum%)   'comparison to host's
    e$ = " Checksum error": GOTO PRINTERROR
  END IF
  GOSUB CLEARCOMMAND
  PRINT "Checksum ok ";
  Checksum% = 0                         'checksum command zeros running total
RETURN

'***************************************************************************
'**** Get digital inputs from the M203                                  ****
'**** CHANGES - LASTDIGINP%                                             ****
'**** CALLS   - SENDPACKET, WAITCHAR, GETCHAR                           ****
'***************************************************************************
GETDIGIN:
'  use OUTB1% = &H80  below for digital input without checksum
'  the checksum is valuable for extracting digital input information
'  from analog data while scanning
  OUTB1% = &H8E                         '&H8E = diginput with checksum
  OUTB2% = &H4C                         'select digital inputs on 203
  GOSUB SENDPACKET
  IF OUTB1% = &H8E THEN NUMCHARS% = 3 ELSE NUMCHARS% = 2
  GOSUB WAITCHAR: GOSUB GETCHAR         'command echo
  IF OUTB1% <> INPBYTE% THEN e$ = " Digital In token wrong ": GOTO PRINTERROR
  GOSUB GETCHAR
  GOSUB CLEARCOMMAND
  LASTDIGINP% = INPBYTE%
  PRINT "Digital in "; DIGINP$(LASTDIGINP%);
  IF OUTB1% = &H8E THEN
    GOSUB GETCHAR
    IF Checksum% <> (INPBYTE% + INPBYTE%) MOD 256 THEN
       e$ = " Digital In Checksum token wrong ": GOTO PRINTERROR
    END IF
  END IF
RETURN

CALCWAIT:
  WAITIME = (SamplePeriod# * TotalPeriods%) + .04 * (BAUD% + 1) + .1
  'time to acquire and transmit one conversion, plus 0.1 second
RETURN


SetPeriod:                          'determine summation period
  GOSUB CLEARCOMMAND
  LOCATE 25, 40
  INPUT ; "Enter rate "; AcqRate
  IF (AcqRate < Maxrate / 255) OR AcqRate > 120 THEN GOTO SetPeriod
  AcqPeriod = 1 / AcqRate: TotalPeriods% = INT(AcqPeriod * Maxrate + .5)
  IF TotalPeriods% = 0 THEN TotalPeriods% = 1
  MODEREGLO% = TotalPeriods%
  GOSUB CALCVARS
  GOSUB SENDADMODE
  XX = WAITIME: GOSUB WAITXX
  GOSUB SYSCAL
  GOSUB CLEARCOMMAND
RETURN

LocalAvg:
  GOSUB CLEARCOMMAND
  LOCATE 25, 40
  INPUT ; "Enter Local averaging "; RunningAvg%
  IF RunningAvg% < 1 OR RunningAvg% > 1000 THEN BEEP: GOTO LocalAvg
  REDIM A#(RunningAvg%)
  GOSUB SYSCAL
  GOSUB CLEARCOMMAND
RETURN

GetVersion:
  OUTB1% = &H86: OUTB2% = 0                      'request Model 203 version
  GOSUB SENDPACKET: NUMCHARS% = 2: GOSUB WAITCHAR        'wait for 2 chars
  GOSUB GETCHAR
  IF INPBYTE% <> &H86 THEN e$ = "version command failure": GOTO PRINTERROR
  GOSUB GETCHAR
  VERSION = INPBYTE%
RETURN

'***************************************************************************
'****  Prompt for channel code, send data to the M203                   ****
'****  CHANGES - CHANNEL%, EXTCCODE%                                    ****
'****  CALLS   - SENDPACKET, CLEARCOMMAND, SETCCODE, CLEARSDCOUNT       ****
'***************************************************************************
CHANNELCHG:                             'send channel select to M203
  GOSUB CLEARCOMMAND
  LOCATE 25, 25: PRINT "Enter Channel:       (select: 0, 1, 6, 7)";
  LOCATE 25, 39: INPUT ; " ", TCHANNEL$
  GOSUB CLEARCOMMAND
  IF TCHANNEL$ = "0" OR TCHANNEL$ = "1" OR TCHANNEL$ = "6" OR TCHANNEL$ = "7" THEN
    GOSUB SETCHANNEL
  ELSE GOTO CHANNELCHG
  END IF
RETURN

SETCHANNEL:
    Channel% = VAL(TCHANNEL$)
SETCCODE:
    ChanChgFlag% = TRUE             'set flag used to zero running avg array
    OUTB1% = &H1                    'channel change command
    OUTB2% = (Channel% * 16)        'Channel% in upper nibble
    GOSUB SENDPACKET
    GOSUB CLEARSDCOUNT
RETURN

'***************************************************************************
'****  Prompt for digital output code, send data to the M203            ****
'****  CHANGES - LASTDIGOUT%                                            ****
'****  CALLS   - SENDPACKET, CLEARCOMMAND                               ****
'***************************************************************************
SETDIGOUT:                              'send digital output code to M203
  OUTB1% = 2                            'digital output token
  GOSUB CLEARCOMMAND
  INPUT ; "Enter Output Code"; OUTB2%
  GOSUB CLEARCOMMAND
  IF OUTB2% < 0 OR OUTB2% > 255 THEN GOTO SETDIGOUT
  LASTDIGOUT% = OUTB2%
  GOSUB SENDPACKET
RETURN

'***************************************************************************
'****  Send warmstart token to the M203, then send mode register data   ****
'****  use after changing rate, SAMPLEPERIOD% or wordlength             ****
'****  USES    - MODEREGHI%, MODEREGMID%, MODEREGLO%, INPBYTE%          ****
'****  CHANGES - NUMCHARS%                                              ****
'****  CALLS   - SENDPACKET, WAITXX, WAITCHAR, GETCHAR, CALCWAIT        ****
'****            READMODE                                               ****
'***************************************************************************
SENDADMODE:
  WAITIME = 2
  OUTB1% = &H84                                 'warm start token
  GOSUB SENDPACKET
  NUMCHARS% = 1
  GOSUB WAITCHAR
  MODEREGMID% = WL%                             'must send latest word length
  OUTB1% = MODEREGHI%: OUTB2% = MODEREGMID%
  GOSUB SENDPACKET
  OUTB1% = MODEREGLO%: OUTB2% = TimeBase%
  GOSUB SENDPACKET
  GOSUB GETCHAR
  IF INPBYTE% <> &H84 THEN e$ = " wrong warm start token": GOTO PRINTERROR
  GOSUB READMODE
  GOSUB CALCWAIT
RETURN

'***************************************************************************
'****  Change scanning type                                             ****
'****  USES    -                                                        ****
'****  CHANGES - SCANINTERVAL#, ScanDisp$                               ****
'****  CALLS   - CALCSCANTIME, DOSCAN, SIGNON, EMPTYBUFFER, SETCCODE    ****
'***************************************************************************
SCANMENU:
  CLS
  POLLEDFLAG% = TRUE
  LOCATE 4, 27: PRINT "SCANNING  CONFIGURATION"
  LOCATE 9, 15: PRINT "   CHANNEL:"
  LOCATE 9, 26: PRINT SINGLECHAN%;
  LOCATE 15, 17: PRINT "C) Channel Select  (0, 1, 6, 7)";
  LOCATE 17, 17
  PRINT USING "I) Scan Interval:  ######.#### seconds"; SCANINTERVAL#;
  LOCATE 19, 17: PRINT "D) Display Mode: "; ScanDisp$; "     ";
  LOCATE 21, 13: PRINT "Enter) Perform the scan"
  LOCATE 23, 15: PRINT "Esc) Back to Polled Mode";
  A$ = UCASE$(INPUT$(1))
  IF A$ = "C" THEN GOSUB GETCHANNEL                     'select remote avg
  'LOCATE 7, 15: PRINT "   CHANNEL:"; SINGLECHAN%
  IF A$ = "I" THEN GOSUB INPUTSCANTIME: SIGNONFLAG% = TRUE
  IF A$ = "D" THEN
    IF ScanDisp$ = "Graphics" THEN ScanDisp$ = "Text" ELSE ScanDisp$ = "Graphics"
  END IF
  IF A$ = CHR$(13) THEN
    IF ScanDisp$ = "Graphics" THEN GOSUB GRAPHICMODESETUP
    GOSUB DOSCAN
    WAITIME = SCANINTERVAL# * 1.2
    IF CHKSUMFLG% THEN WORDCNT% = WORDCNT% + 1
    POLLEDFLAG% = FALSE: running% = TRUE
    IF ScanDisp$ = "Graphics" THEN
	GOSUB STARTGRAPH
    ELSE
	GOSUB TEXTSCANSTART
    END IF
    IF CHKSUMFLG% THEN WORDCNT% = WORDCNT% - 1
    GOSUB CALCWAIT
    GOTO SCANMENU
  END IF
  IF A$ = CHR$(27) THEN
    LOCATE 25, 10: PRINT "           Clearing serial buffer"; SPACE$(20);
    GOSUB EMPTYBUFFER
    GOSUB SETCCODE
    RETURN
  END IF
  IF VALA% > 8 AND A$ <> "C" THEN SOUND 1500, 1
  GOTO SCANMENU

CANCELTRIGGER:
  SCREEN 0: A$ = ""
  GOSUB EMPTYBUFFER
  GOSUB SENDMRESET
  GOSUB SIGNON
GOTO SCANMENU

'*****************************************************************************
'The A/D is constantly performing conversions at RATE. Slower RATEs
'provide lower noise, higher RATEs allow higher frequencies to be seen,
'and yield higher resolution in the time domain.
'
'For higher frequency applications the RATE should be set for the
'appropriate low-pass filter cutoff frequency. The cutoff frequency is
'approximately one quarter the MAX RATE WITH LOCAL AVERAGING AT 2.
'
'

'***************************************************************************
'****  SCANVALS-INPUTSCANTIME-CALCSCANTIME                              ****
'****  Calculates Min/Max scanning rate, prompts for a new rate, and    ****
'****  calculates the # of rollovers for that rate                      ****
'****  USES    - BAUD%, RATE, WORDCNT%                                  ****
'****  CHANGES - MAXTIME#, MINTIME#, CPS, SCANINTERVAL#                 ****
'****            SECROLL#, ROLLSCAN#, SCAN%, SCAN1%, SCAN2%, AVERAGE    ****
'***************************************************************************

SCANVALS:
  GOSUB CALCVARS
  SECROLL# = (1 / (PICCYCLETIME# * 1024)) / 2 ^ BAUD% 'rollovers/sec

  TEMPCNT% = WORDCNT%                            'TEMPCNT% is used locally
  IF CHKSUMFLG% THEN TEMPCNT% = TEMPCNT% + 1
  MAXTIME# = (65535 / SECROLL#)                  'max scan interval
'Determine conversion time and sample transmission times for current
'  data rate and baud rate settings           
  MINTIME# = SamplePeriod# * TotalPeriods%  'Time required for each conversion
  BAUDTIME# = TEMPCNT% * (2 ^ BAUD% / 960)  ' Time required for sample to be transferred
  
'Determine which interval is greater and assign to MINTIME#
  IF MINTIME# < BAUDTIME# THEN MINTIME# = BAUDTIME#

'Adjust MINTIME# for integer rounding
  SAVESCANINT# = SCANINTERVAL#           ' Save current scan interval
  SCANINTERVAL# = MINTIME#
  GOSUB ROUNDIT
  MINTIME# = SCANINTERVAL#
  SCANINTERVAL# = SAVESCANINT#
  GOSUB ROUNDIT
  MAXTIME# = INT(MAXTIME# * SECROLL#) / SECROLL#
RETURN


INPUTSCANTIME:
  CLS : GOSUB SCANVALS
INPUT2:
  LOCATE 10, 10: PRINT "Enter scan interval ";
  PRINT USING "#####.####  - "; MINTIME#;
  PRINT USING "#######.####  seconds"; MAXTIME#;
  LOCATE 12, 10
  PRINT USING "Current scan interval: ##.#### "; SCANINTERVAL#
  LOCATE 10, 66
  INPUT T$: TEMPINPUT# = VAL(T$)                'get user input
  IF TEMPINPUT# = 0 THEN RETURN                 'return on no user input
  SCANINTERVAL# = TEMPINPUT#
  GOTO CALC2                                    'validate new interval

CALCSCANTIME:
  GOSUB SCANVALS
CALC2:
  GOSUB ROUNDIT
  IF (SCANINTERVAL# > MAXTIME#) OR SCANINTERVAL# < MINTIME# * .98 THEN
    CLS
    LOCATE 15, 10: PRINT "Scan interval out of limits, press any key to continue"
    A$ = INPUT$(1)
    GOTO INPUTSCANTIME
  END IF
  A$ = ""
RETURN

ROUNDIT:                                   
  ROLLSCAN# = SECROLL# * (SCANINTERVAL# + .00005) - 1  'rollovers per scan
  SCAN2% = INT(ROLLSCAN# / 65536)
  SCAN2# = SCAN2%                  'for current version, SCAN2% is always 0
  ROLLSCAN# = ROLLSCAN# - (SCAN2# * 65536)           'split into three bytes
  SCAN1% = INT(ROLLSCAN# / 256)
  SCAN1# = SCAN1%
  ROLLSCAN# = ROLLSCAN# - (SCAN1# * 256)
  SCAN% = INT(ROLLSCAN#)
  SCANINTERVAL# = ((SCAN% + SCAN1# * 256 + SCAN2# * 65536 + 1) / SECROLL#)
RETURN

'***************************************************************************
'****  Get input channel routine variation, used from scan menu only    ****
'****  USES    - VALA%                                                  ****
'****  CHANGES - SINGLECHAN%,                                           ****
'***************************************************************************
GETCHANNEL:
  LOCATE 10, 18: PRINT SPACE$(40);
  LOCATE 10, 18: INPUT ; "Enter Channel: ", TCHANNEL%
  IF TCHANNEL% = 0 OR TCHANNEL% = 1 OR TCHANNEL% = 6 OR TCHANNEL% = 7 THEN
    SINGLECHAN% = TCHANNEL%
    LOCATE 10, 18: PRINT SPACE$(40);
    ChanChgFlag% = TRUE                   'set flag used to zero A# array
    OUTB1% = &H1                          'channel change command
    OUTB2% = SINGLECHAN% * 16             'channel number * 16
    GOSUB SENDPACKET
    GOSUB CLEARSDCOUNT
  ELSE
    GOTO GETCHANNEL
  END IF
RETURN

'***************************************************************************
'****  Write the setup variables to the file DRIVR203.CFG in the        ****
'****  default directory                                                ****
'***************************************************************************
SAVECONFIG:
  ON ERROR GOTO WRITERR
  OPEN "DRIVR203.CFG" FOR OUTPUT AS #2
  WRITE #2, MODEREGHI%, MODEREGMID%, MODEREGLO%, Channel%, SUBSAMPLE%, SPARE%
  WRITE #2, DACENABLE%, BAUD%, COM$, INPBUF%, SPARE%, BU%, GAIN%, Maxrate, WL%
  WRITE #2, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%
  WRITE #2, PLACEHOLDER%, PLACEHOLDER%, SINGLECHAN%
  WRITE #2, SCAN%, SCAN1%, SCAN2%, SCANINTERVAL#, STYPE%, RESERVE%
  WRITE #2, Dac1Volts, Dac2Volts, TotalPeriods%, ScanDisp$, POSFS, NEGFS, VDIV, HDIV
  WRITE #2, GAIN#, RunningAvg%
  CLOSE : ON ERROR GOTO 0
RETURN
WRITERR:
   CLS : PRINT "Unable to save DRIVR203.CFG file, press any key to continue"
   A$ = INPUT$(1)
RESUME 3000
3000 ON ERROR GOTO 0: RETURN

'***************************************************************************
'****  Read the setup variables from the DRIVR203.CFG file in the       ****
'****  default directory                                                ****
'****  CALLS   -                                                        ****
'***************************************************************************
READCONFIG:
' model 203 uses DRIVR203.CFG
  ON ERROR GOTO READERR
  OPEN "DRIVR203.CFG" FOR INPUT AS #2
  INPUT #2, MODEREGHI%, MODEREGMID%, MODEREGLO%, Channel%, SUBSAMPLE%, SPARE%
  INPUT #2, DACENABLE%, BAUD%, COM$, INPBUF%, SPARE%, BU%, GAIN%, Maxrate, WL%
  INPUT #2, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%
  INPUT #2, PLACEHOLDER%, PLACEHOLDER%, SINGLECHAN%
  INPUT #2, SCAN%, SCAN1%, SCAN2%, SCANINTERVAL#, STYPE%, RESERVE%
  INPUT #2, Dac1Volts, Dac2Volts, TotalPeriods%, ScanDisp$, POSFS, NEGFS, VDIV, HDIV
  INPUT #2, GAIN#, RunningAvg%
  CLOSE

4000
  ON ERROR GOTO 0
  MODEREGMID% = WL%: WORDCNT% = 3      ' always use 24-bit mode 9/29
  Div% = 0                             ' do not use remote division
RETURN

'***************************************************************************
'****  Set default values if configuration file cannot be located       ****
'***************************************************************************

READERR:
  CLS : PRINT "Unable to read DRIVR203.CFG file, press any key to continue"
  A$ = INPUT$(1): COM$ = "COM1:": ScanDisp$ = "Text"
  Channel% = 7: SINGLECHAN% = 0
  GAIN% = 1: GAIN# = 1
  TotalPeriods% = 1: Div% = 0
  RunningAvg% = 4
  Maxrate = 120: SCANINTERVAL# = 1
  Dac1Volts = 0: Dac2Volts = 0
  INPBUF% = 32767
  WL% = 128
  BAUD% = 0                     'Baud rate code can be 0 to 5, default 9600
  POSFS = 5: NEGFS = -5         'positive and negative full scale (in volts)
  VDIV = 1                      'vertival index (in volts)
  HDIV = 0                      'horizontal index is set by GETHINDEX
  PLACEHOLDER% = 0              'used in config file for version compatibility
  RESERVE% = 0                  'reserved for future use
  MODEREGLO% = 1
RESUME 4000

'***************************************************************************
'****  Clears the polled-mode screen command line                       ****
'***************************************************************************
CLEARCOMMAND:
  LOCATE 25, 2: PRINT SPACE$(79); : LOCATE 25, 51
RETURN

'***************************************************************************
'****  Check serial communications with the M203                        ****
'****  USES    - INPBYTE%, OUTBYTE%                                     ****
'****  CHANGES - WAITIME, NUMCHARS%                                     ****
'****  CALLS   - WAITCHAR, GETCHAR                                      ****
'***************************************************************************
ECHOTEST:
  OUTBYTE% = 1: I = 1: ERRCNT = 0: CLS
  WAITIME = 1: NUMCHARS% = 1
  LOCATE 10, 20: PRINT "BYTES SENT              ERRORS"
  LOCATE 12, 46: PRINT "0"
  LOCATE 25, 20: PRINT "Press any key to continue";
ECHOLOOP:
  PRINT #1, CHR$(OUTBYTE%); : GOSUB WAITCHAR    'wait for 1 char, 1 sec
  LOCATE 12, 28: GOSUB GETCHAR: LOCATE 12, 20
  PRINT USING "##########               "; I;
  IF INPBYTE% <> OUTBYTE% THEN BEEP: ERRCNT = ERRCNT + 1: PRINT ERRCNT
  I = I + 1                                     'byte count
  OUTBYTE% = (OUTBYTE% + 1) MOD 255
  IF OUTBYTE% = 0 THEN OUTBYTE% = 1             'skip 0, it's the end echo token
  IF INKEY$ = "" THEN GOTO ECHOLOOP
RETURN

'***************************************************************************
'****  Exit choices                                                     ****
'****  CALLS   - SAVECONFIG                                             ****
'***************************************************************************
EXITPROGRAM:
  CLS : LOCATE 12, 20: PRINT "SAVE CONFIGURATION DATA ? (Y/N) "
  A$ = UCASE$(INPUT$(1))
  IF A$ = "Y" THEN GOSUB SAVECONFIG: GOTO EXITPROGRAM1
  IF A$ <> "N" THEN GOTO EXITPROGRAM
EXITPROGRAM1:
  CLS : LOCATE 12, 20: PRINT "EXIT PROGRAM ? (Y/N) "
  A$ = UCASE$(INPUT$(1))
  IF A$ = "N" THEN RETURN
  IF A$ <> "Y" THEN GOTO EXITPROGRAM1
  END    'leave the program

'***************************************************************************
'****  Get a char from the serial port, update checksum                 ****
'****  USES    - LOC(1)                                                 ****
'****  CHANGES - CHECKSUM%                                              ****
'***************************************************************************
GETCHAR:
  IF LOC(1) <> 0 THEN INPBYTE% = ASC(INPUT$(1, #1)) ELSE BEEP
  Checksum% = (Checksum% + INPBYTE%) MOD 256
RETURN

'***************************************************************************
'****  Display M203 error handler                                       ****
'****  USES   - INPBYTE%, E$, LOC(1)                                    ****
'****  CHANGES - A$                                                     ****
'***************************************************************************
PRINTERROR:
  BEEP
  PRINT : PRINT " error   "; e$
  PRINT "after "; OUTB1%; " command"
  PRINT " at "; TIME$; " on "; DATE$
  XX = 1: GOSUB WAITXX
  PRINT LOC(1); " byte(s) in input buffer"
  IF LOC(1) = 0 THEN GOTO PRINTLAST
  bytes = LOC(1)
  FOR N = 1 TO bytes
    INPBYTE% = ASC(INPUT$(1, #1))
    PRINT " byte "; N; "="; INPBYTE%; "( or, in hex) "; HEX$(INPBYTE%)
  NEXT N
  PRINT
PRINTLAST:
  PRINT " Last received byte may have been an error code."
  PRINT " The last byte was "; INPBYTE%; "  ";
  IF INPBYTE% = 1 THEN PRINT "ChkSum error"
  IF INPBYTE% = 2 THEN PRINT "CmdPending error"
  IF INPBYTE% = 3 THEN PRINT "Master reset code"
  IF INPBYTE% = 5 THEN PRINT "sign-on error"
  IF INPBYTE% = 6 THEN PRINT "Baud error"         '7 is unused
  IF INPBYTE% = 8 THEN PRINT "Pass through error"
  IF INPBYTE% = 9 THEN PRINT "Data request error"
ERRKEY:
PRINT : PRINT " Press Q to restart. ";
  PRINT " (Your last-saved .CFG file will be read from the disk.)"
  A$ = UCASE$(INPUT$(1))
  IF A$ = "Q" THEN RUN ELSE GOTO ERRKEY

'***************************************************************************
'****  Send master reset to M203                                        ****
'***************************************************************************
SENDMRESET:
  OUTB1% = &H0           'master reset token
  PRINT #1, CHR$(OUTB1%);
RETURN

'***************************************************************************
'****  Send three bytes to M203. The third byte is always a checksum    ****
'****  USES    -  OUTB1%, OUTB2%                                        ****
'****  CHANGES -  CHKSUM%                                               ****
'***************************************************************************
SENDPACKET:
  PRINT #1, CHR$(OUTB1%);
  PRINT #1, CHR$(OUTB2%);
  CHKSUM% = ((OUTB1% + OUTB2%) AND 255)
  PRINT #1, CHR$(CHKSUM%);
RETURN

'***************************************************************************
'****   Wait XX seconds, best for delays under 10 seconds               ****
'****  USES    - TIMER, XX                                              ****
'****  CHANGES - XX                                                     ****
'***************************************************************************
WAITXX:
  IF TIMER + XX > 86399 THEN GOTO WAITXX     'wait for end of day rollover
  XX = TIMER + XX
WAITLOOP:
  IF TIMER < XX THEN GOTO WAITLOOP
RETURN

'***************************************************************************
'****  Wait for NUMCHARS% char's in serial buffer, or WAITIME seconds   ****
'****  USES    - TIMER, WAITTIME, NUMCHARS%                             ****
'****  CHANGES - CHECKKEY%, CANCELFLAG%                                 ****
'***************************************************************************
WAITCHARORKEY:        'use this entry point to Return after any keystroke
CHECKKEY% = TRUE: GOTO WAITCHAR1
WAITCHAR:
  CHECKKEY% = FALSE
WAITCHAR1:
  CANCELFLAG% = FALSE
  IF TIMER + WAITIME > 86399 THEN GOTO WAITCHAR1        'wait for end of day
  XX = TIMER + WAITIME + .1                             'time to wait
HOLDON:
 
  IF CHECKKEY% THEN
    IF toggle$ = "*" THEN toggle$ = " " ELSE toggle$ = "*"
    PRINT toggle$; : LOCATE , 1
    A$ = UCASE$(INKEY$)
    IF A$ <> "" THEN
	XX = WAITIME * 1.2: GOSUB WAITXX
	GOSUB EMPTYBUFFER
	CANCELFLAG% = TRUE
	GOTO EXITWAITCHAR
    END IF
  END IF
  IF (TIMER < XX) AND (LOC(1) < NUMCHARS%) THEN GOTO HOLDON
    IF (LOC(1) < NUMCHARS%) THEN
	CLS : PRINT "only "; LOC(1);
	PRINT " out of "; NUMCHARS%; " chars received"
	' BASIC can lock-up after a missed interrupt
	' disabling and reenabling interrupts fixes the problem
	GOSUB LOSTINTERRUPT
	e$ = " Serial port time-out": GOTO PRINTERROR
  END IF
EXITWAITCHAR:
RETURN

'***************************************************************************
'****  Send master reset token to the M203                              ****
'****  USES    -  INPBYTE%                                              ****
'****  CHANGES -                                                        ****
'****  CALLS   -  SENDMRESET, WAITXX, EMPTYBUFFER, PRINTERROR           ****
'***************************************************************************
DOMRESET:
  PRINT "         Reset           ";
  GOSUB SENDMRESET
  XX = .5: GOSUB WAITXX
  PRINT "Clearing serial buffer"; SPACE$(20);
  GOSUB EMPTYBUFFER
  IF INPBYTE% = 3 THEN GOTO EXITDOMRESET  'correct return code
  e$ = " M203 did not respond correctly to Master Reset"
  GOTO PRINTERROR
EXITDOMRESET:
RETURN


'***************************************************************************
'****  Perform a scan                                                   ****
'****  USES    - SINGLECHAN%                                            ****
'****            WORDCNT%, LOC(1), TIMER, SCAN%, SCAN1%, SCAN2%         ****
'****            CHAN0%, CHAN1%, CHAN2%, CHAN3%, CHAN4%, CHAN5%         ****
'****  CHANGES - CHAN0%, CHAN1%, CHAN2%, CHAN3%, CHAN4%, CHAN5%         ****
'****            COMPLETESCANS                                          ****
'****  CALLS   - WAITXX, SYSCAL, WAITCHAR, GETCHAR, DISPLAYSCAN, SIGNON ****
'****            DOMRESET, INPUTSCANTIME, SETMODEREG, SENDPACKET        ****
'***************************************************************************
DOSCAN:
  IF (WORDCNT% + 100) > INPBUF% THEN
    CLS : LOCATE 12, 17
    PRINT "Input buffer too small, press any key to continue"
    A$ = INPUT$(1): RETURN SCANMENU
  END IF
  GOSUB CALCSCANTIME
  IF SIGNONFLAG% THEN           'new SCANINTERVAL# sets SIGNONFLAG%
    GOSUB SIGNON
    GOSUB INITDACS
  END IF
  GOSUB SYSCAL
  SCANWORDS = WORDCNT%
  OUTB1% = &H1                                          'channel code command
  OUTB2% = SINGLECHAN% * 16
  GOSUB SENDPACKET
  ChanChgFlag% = TRUE                                   'flag the channel change
  XX = .1: GOSUB WAITXX                                 'hardware settling time
     'pick the correct startscan token out of four chouces
  IF TRGRSCANFLG% THEN
    IF CHKSUMFLG% THEN OUTB1% = &H8C ELSE OUTB1% = &H89
    IF EDGE% = 1 THEN TRGR$ = "POSITIVE":  ELSE TRGR$ = "NEGATIVE"
    OUTB2% = EDGE%
  ELSE
    IF CHKSUMFLG% THEN OUTB1% = &H8D ELSE OUTB1% = &H8B
  END IF


  GOSUB SENDPACKET
  NUMCHARS% = 1: GOSUB WAITCHAR


  GOSUB GETCHAR                                 'remove start scan token
  IF INPBYTE% <> OUTB1% THEN e$ = "Scan start failure": GOTO PRINTERROR
  CLS
  IF TRGRSCANFLG% THEN
    LOCATE 12, 1: PRINT "WAITING FOR A "; TRGR$; " TRIGGER";
    LOCATE 25, 1: PRINT "Press ESC to exit";

TRGRWAIT:                                
    IF INKEY$ = CHR$(27) THEN RETURN CANCELTRIGGER
    IF LOC(1) = 0 THEN GOTO TRGRWAIT
  END IF

  XX = MINTIME#: GOSUB WAITXX                   'wait one conversion time
  SCANSTART = TIMER                             'the first scan begins now
  STARTTIME$ = TIME$
  STARTDAY$ = DATE$: DAYS = 0
  SCAN& = 0
RETURN


TEXTSCANSTART:
  CLS : LOCATE 25, 24
  PRINT "Press any key to end scan."
  LOCATE 8, 10: PRINT "Scanning started at              "; STARTTIME$
  LOCATE 10, 10
  PRINT USING "Scanning interval, seconds #########.####"; SCANINTERVAL#
  LOCATE 12, 10
  PRINT USING "Bytes expected per scan       ######"; WORDCNT%
SCANSTATUS:
  IF STARTDAY$ <> DATE$ THEN DAYS = DAYS + 86400: STARTDAY$ = DATE$
  TEMP = LOC(1)
  IF running% THEN elapsed = TIMER + DAYS - SCANSTART ELSE TEMP = TEMP - 1
  COMPLETESCANS = TEMP / WORDCNT%
  LOCATE 14, 10
  PRINT USING "Elapsed time, seconds       ########"; INT(elapsed)
  Expt% = INT(elapsed / SCANINTERVAL#) + 1
  LOCATE 16, 10
  PRINT USING "Scans expected             #########"; Expt%
  LOCATE 18, 10
  PRINT USING "Scans completed            #########.##"; COMPLETESCANS
  IF NOT running% THEN
    TEMP = LOC(1)
    XX = .3: GOSUB WAITXX
    IF LOC(1) = TEMP THEN
	GOTO EXITDOSCAN
    ELSE GOTO SCANSTATUS
    END IF
  END IF
  A$ = INKEY$                                   'check for a keystroke
  IF LOC(1) > (INPBUF% - 100) THEN
    GOSUB STOPSCAN  'stop immediately on buff overflow
    LOCATE 20, 25: PRINT "Buffer full. Press space to continue";
    A$ = INPUT$(1): A$ = ""
    running% = FALSE: GOTO SCANSTATUS
  END IF
  IF A$ = "" THEN GOTO SCANSTATUS               'no keystroke, buffer not full
  IF running% THEN                              'if keystroke, then
    GOSUB STOPSCAN                              ' stop the scan
  END IF
  GOTO SCANSTATUS

EXITDOSCAN:
  GOSUB DISPLAYSCAN
RETURN

STOPSCAN:
  OUTB1% = &H8A: GOSUB SENDPACKET               'send end scan token
  XX = SCANINTERVAL# + .1: GOSUB WAITXX
  LOCATE 25, 1: PRINT SPACE$(45);
  LOCATE 25, 30: PRINT "Scanning will stop.";
  running% = FALSE
RETURN

'***************************************************************************
'****  Display scan data                                                ****
'****  USES    - COMPLETESCANS, SCANWORDS, INPBYTE%,                    ****
'****            WORDCNT%                                               ****
'****  CHANGES - SCANS, NUMCHARS%                                       ****
'****  CALLS   - GETCHAR, VOLTSCALE, VOLTDISPLAY, WAITCHAR, GCONV       ****
'****            GETCHKSUM                                              ****
'***************************************************************************
DISPLAYSCAN:
IF INKEY$ <> "" THEN GOTO DISPLAYSCAN                 'clear keybrd buffer
  POLLEDFLAG% = FALSE: CLS
  LOCATE 12, 1
  PRINT "CHANNEL: "; SINGLECHAN%
  NUMCHARS% = WORDCNT%

SCANLOOP:
  IF SCAN& = INT(COMPLETESCANS) THEN
    SCAN& = 0
    GOTO SCANSDONE
  END IF
  GOSUB GetConv
  LOCATE 12, 20
  PRINT USING "VOLTS:  ##.######"; volts
  SCAN& = SCAN& + 1
  LOCATE 25, 1: PRINT USING "SCAN ######  of"; SCAN&;
  PRINT INT(COMPLETESCANS);
  PRINT "  Press Esc to quit          ";
  A$ = INPUT$(1)
  IF A$ = CHR$(27) THEN SCAN& = 0: GOTO EXITSCANSDONE
  GOTO SCANLOOP

SCANSDONE:
  CLS
  IF (COMPLETESCANS = 0) OR (COMPLETESCANS <> INT(COMPLETESCANS)) THEN
    LOCATE 10, 10: PRINT "Last scan was incomplete, press any key to continue"
    A$ = INPUT$(1)
  ELSE
    WHILE LOC(1) > 1                    'go to end of buff for end-scan token
	GOSUB GETCHAR
    WEND
    GOSUB GETCHAR
    IF INPBYTE% <> &H8A THEN            'check for end of scanning token &H8A
	PRINT "No end scan token, press any key to continue"
	A$ = INPUT$(1)
    END IF
  END IF
EXITSCANSDONE:
  GOSUB EMPTYBUFFER   'it is possible that another scan is in the buffer
RETURN

'***************************************************************************
'**** ------------  Additional programmer's  information -------------  ****
'****               Command tokens are hexidecimal                      ****
'***************************************************************************
'
'  INPUT COMMANDS
'       These are always sent as the first byte of a three-byte packet
'       Input command tokens are always echoed by the Model 203
'
'       80 = get digital input bit without checksum  (OUTB2% = &H4C)
'       81 = get a conversion result
'       84 = warmstart, sets timing
'       86 = get version
'       87 = get checksum
'       89 = start a scan, with trigger without checksum
'       8A = end a scan
'       8B = start a scan, without trigger or checksum
'       8C = start a scan, with trigger and checksum
'       8D = start a scan, without trigger with checksum
'       8E = digital input with checksum     (OUTB2% = &H4C)

'   RESET COMMAND      Reset is the only single-byte command
'        0  = master reset and end echo

'   OUTPUT COMMANDS
'        These are always sent as the first byte of a three-byte packet
'        Output command tokens are not echoed

'        1  = select A/D channel
'        2  = set external optically isolated output port
'        4  = change averaging
'        5  = set analog output timebase
'        8  = set analog output low byte for either analog output
'        9  = set analog output high byte and update either output
'             determined by msb of high byte

'   OTHER
'       88 = signon token, (not a command)

'***************************************************************************
'****  Initialize DACS VOLTAGE and SETS DAC TIMEBASE                                                 ****
'****  USES    - Dac1Volts, Dac2Volts, ADUTYCYLCE%, MAXCNT%, VOLTAGE  ****
'****            ANALOW%, ANAHIGHT%                                     ****
'****  CHANGES -                                                        ****
'****  CALLS   - SENDPACKET                                             ****
'***************************************************************************
INITDACS:
  OUTB1% = 5
  OUTB2% = INT((MAXCNT% / 256) + 1)
  GOSUB SENDPACKET
  Dac$ = "1"
  VOLTAGE = Dac1Volts
  GOSUB SetDAC
  Dac$ = "2"
  VOLTAGE = Dac2Volts
  GOSUB SetDAC
RETURN

'***************************************************************************
'****  Change current DAC settings                                      ****
'****  USES    - ADUTYCYLCE%, MAXCNT%, VOLTAGE, ANALOW%, ANAHIGHT%      ****
'****            TIMEBASECNT                                            ****
'****  CHANGES - Dac1Volts, Dac2Volts                                 ****
'****  CALLS   - SENDPACKET                                             ****
'***************************************************************************
ANALOGOUT:
   CLS
   LOCATE 10, 30: PRINT "DAC 1          DAC 2";
   LOCATE 12, 30: PRINT USING "#.####         "; Dac1Volts; Dac2Volts
   LOCATE 25, 15: PRINT "1 for DAC 1, 2 for DAC 2, Esc to return";
   PRINT "    Resolution  1 /"; MAXCNT% + 1;
GetDAC:
   Dac$ = UCASE$(INPUT$(1))
   IF Dac$ = "" THEN GOTO GetDAC
   IF Dac$ = CHR$(27) THEN GOTO EXITANALOGOUT
   IF Dac$ <> "1" AND Dac$ <> "2" THEN GOTO GetDAC
ENTERV:
   CLS : LOCATE 10, 20: PRINT "Enter DAC "; Dac$; " output voltage (0 TO 5) ";
   INPUT VOLTAGE
   IF VOLTAGE < 0 OR VOLTAGE > 5 THEN GOTO ENTERV
SetDAC:
   IF VOLTAGE < 0 THEN VOLTAGE = 0
   IF VOLTAGE > 5 THEN VOLTAGE = 5
   ADUTYCYCLE% = MAXCNT% * VOLTAGE / 5          'get duty cycle, and split
   ANAHIGH% = INT(ADUTYCYCLE% / 256)            'into high and low bytes
   ANALOW% = INT(ADUTYCYCLE% - ANAHIGH% * 256)
   OUTB1% = 8                                   'token for low byte
   OUTB2% = ANALOW%
   GOSUB SENDPACKET                             'send low byte
   OUTB1% = 9                                   'token for high byte
   IF Dac$ = "1" THEN
     Dac1Volts = VOLTAGE
     OUTB2% = ANAHIGH% AND &H7F   'pass high byte value. msb won't be set
   ELSE
     Dac2Volts = VOLTAGE          'this must be second DAC
     OUTB2% = ANAHIGH% OR &H80    'pass high byte value, with msb set
   END IF
   GOSUB SENDPACKET
   IF A$ = "A" THEN GOTO ANALOGOUT
EXITANALOGOUT:
RETURN

'***************************************************************************
'****  Calibrate full-scale to user input value                         ****
'****  USES    - FSREADING, Total&, OFFSET&                             ****
'****  CHANGES - GAIN#                                                  ****
'****  CALLS   - CLEARCOMMAND                                           ****
'***************************************************************************
FullScaleCalToValue:
   GOSUB CLEARCOMMAND
   LOCATE 25, 1
   INPUT "ENTER FULL SCALE READING (MV) "; FSREADING
   GAIN# = (FSREADING / 1000) / (Total& - OFFSET&)
   A$ = " "
RETURN

' the following sample routine is used to illustrate that analog output
' commands can be sent at any time during scanning
' a sawtooth waveform is generated at Dac1
CHANGEVOLTS:
    VOLTAGE = VOLTAGE + .002: IF VOLTAGE > 5 THEN VOLTAGE = 0
    Dac$ = "1"
    GOSUB SetDAC
RETURN


'**************** MODEL203 ************************************
'***     G R A P H I C S   R O U T I N E S                  ***
'**************************************************************

GRAPHICMODESETUP:
  ON ERROR GOTO NoVGA
   SCREEN 12      '640 x 400  = horizontal, vertical 1,1 = upper left
  ON ERROR GOTO 0 'RECOVER
   PALETTE 3, (0 * 65536 + 35 * 256 + 63)  'create orange color, color 3
RETURN

NoVGA:
  CLS : PRINT " VGA Video required": END

STARTGRAPH:
 GOSUB SETUPSCREEN                      'set up graphic screen
FIRSTPOINT:
  FIRSTSCAN% = -1                       'flag first point on screen
  SCOLOR% = 10                          'select color scan
  LINE (S%, 0)-(S%, 399), 0
  WHILE S% <> 639                       'while not to right edge of screen
WAITFORONESCAN:
    IF LOC(1) > (INPBUF% - 100) THEN
	LOCATE 30, 35: COLOR 15: PRINT "Buffer full"; : COLOR 7
	GOTO ENDSCAN
    END IF
    A$ = UCASE$(INKEY$): COLOR 7
    IF A$ = "E" OR A$ = CHR$(27) THEN A$ = "": GOTO EXITSCAN 'press "E" to end
    IF A$ <> "" THEN GOSUB DOCOMMAND2
    'This line demonstrates the use of analog output commands during scanning
    IF LOC(1) < 2 AND NOT DIGINFLAG% THEN GOSUB CHANGEVOLTS  'for testing
KEYPROCESSED:
    IF (LOC(1) \ SCANWORDS) < 1 THEN GOTO WAITFORONESCAN
    S% = S% + 1: SCAN& = SCAN& + 1
    FOR K% = 0 TO 5                             'draw vertical black bar
	LINE (S% + K%, 0)-(S% + K%, 399), 0
	 IF (S% + K%) / HINDEX% = INT((S% + K%) / HINDEX%) THEN 'redraw
	  FOR NK% = 0 TO 399 STEP VINDEX&: PSET (S% + K%, NK%), 15: NEXT NK%
	END IF
    NEXT K%
    GOSUB DISPSINGLE
    FIRSTSCAN% = 0                      'flag not first point on screen
  WEND                                  'continue until right edge of screen
  S% = 1                                'reset counter screen position counter
  LINE (S%, 0)-(S%, 399), 0             'draw vertical black line
  GOTO FIRSTPOINT                       'wrap to left edge of screen


'*********************************************************************
'*     duplicate of DOCOMMAND above, with differences for graphics   *
'*     plus graph formatting input commands                          *
'*********************************************************************
DOCOMMAND2:
  IF A$ = "C" THEN
INVALIDCHANNEL:
    GOSUB CLEARMENU
    LOCATE 27, 36
    PRINT "Enter Channel:    (select: 0, 1, 6, 7)"
    LOCATE 27, 50
    INPUT ; " ", TCHANNEL$
    IF TCHANNEL$ = "0" OR TCHANNEL$ = "1" OR TCHANNEL$ = "6" OR TCHANNEL$ = "7" THEN
	SINGLECHAN% = VAL(TCHANNEL$)
	GOSUB SETCHANNEL
    ELSE GOTO INVALIDCHANNEL
    END IF
  COLOR 7: LOCATE 27, 24: PRINT "hannel"; SINGLECHAN%;
    GOSUB CLEARMENU
  END IF
  IF A$ = "D" THEN
    IF DIGINFLAG% = FALSE THEN
	DIGINFLAG% = TRUE
	OUTB1% = &H8E                            '&H8E = diginput with chcksum
	OUTB2% = &H4C                            'select digital inputs on 203
	GOSUB SENDPACKET
    END IF
  END IF
  IF A$ = "H" THEN
    GOSUB CLEARMENU
    LOCATE 27, 41: PRINT "Enter horizontal index (seconds) ";
    GOSUB GETINPUT: HDIV$ = KEYINPUT$
    IF HDIV$ <> "" AND VAL(HDIV$) > 0 THEN      'reset screen with
	HDIV = VAL(HDIV$): RETURN STARTGRAPH      'new horizontal index
    ELSE
	GOSUB CLEARMENU
    END IF
  END IF
  IF A$ = "M" THEN
    GOSUB CLEARMENU
    LOCATE 27, 41: PRINT "Enter negative full scale (mv) ";
    GOSUB GETINPUT: NEGFS$ = KEYINPUT$
    IF NEGFS$ <> "" THEN         'reset screen with new negative full scale
	IF VAL(NEGFS$) < (POSFS * 1000) THEN
	  NEGFS = VAL(NEGFS$) / 1000: RETURN STARTGRAPH
	END IF
    END IF
    GOSUB CLEARMENU
  END IF
  IF A$ = "P" THEN
    GOSUB CLEARMENU
    LOCATE 27, 41: PRINT "Enter positive full scale (mv) ";
    GOSUB GETINPUT: POSFS$ = KEYINPUT$
    IF POSFS$ <> "" THEN          'reset screen with new positive full scale
	IF VAL(POSFS$) > (NEGFS * 1000) THEN
	  POSFS = VAL(POSFS$) / 1000: RETURN STARTGRAPH
	END IF
    END IF
    GOSUB CLEARMENU
  END IF
  IF A$ = "U" THEN                      'set the digital outputs
    GOSUB CLEARMENU
SETDIGOUT1:
    LOCATE 27, 41
    INPUT ; "Enter Output Code"; OUTB2%
    GOSUB CLEARMENU
    IF OUTB2% < 0 OR OUTB2% > 255 THEN GOTO SETDIGOUT1
    OUTB1% = 2                           'digital output token
    LASTDIGOUT% = OUTB2%
    GOSUB SENDPACKET
    LOCATE 30, 39
    PRINT LASTDIGOUT%; " ";
  END IF
  IF A$ = "V" THEN
    GOSUB CLEARMENU
    LOCATE 27, 41: PRINT "Enter vertical index (mv) ";
    GOSUB GETINPUT: VDIV$ = KEYINPUT$
    IF VDIV$ <> "" AND VAL(VDIV$) > 0 THEN            'reset screen with
	VDIV = VAL(VDIV$) / 1000: RETURN STARTGRAPH     'new vertical index
    ELSE
	GOSUB CLEARMENU
    END IF
  END IF
  IF A$ = "A" THEN
    GOSUB CLEARMENU
    LOCATE 27, 50
    INPUT ; "Enter #DAC: ", Dac$
    GOSUB CLEARMENU
    LOCATE 27, 50
    INPUT ; "Enter voltage: ", VOLTAGE
    GOSUB CLEARMENU
    A$ = ""
    GOSUB SetDAC
  END IF
  A$ = ""
  RETURN

ENDSCAN:
  A$ = INPUT$(1)
EXITSCAN:
  GOSUB STOPSCAN
  GOSUB EMPTYBUFFER
  SCREEN 0                               'reset screen text mode 0
RETURN

'************************************************************************
'*            error handler for dropped interrupt                       *
'************************************************************************

RECOVER:
GOSUB LOSTINTERRUPT
RESUME FIRSTPOINT

LOSTINTERRUPT:
  ' if your COM ports are at other addresses, change the next line
  IF COM$ = "COM1:" THEN PORTADDRESS = &H3F9 ELSE PORTADDRESS = &H2F9
  TEMP = INP(PORTADDRESS): OUT PORTADDRESS, 0: OUT PORTADDRESS, TEMP
RETURN

'************************************************************************
'*          display single channel on graph                             *
'************************************************************************

DISPSINGLE:                                  
  GOSUB GetConv                   'get reading
  V = volts
  IF V > POSFS THEN V = POSFS ELSE IF V < NEGFS THEN V = NEGFS
  volts% = (V - POSFS) * (399 / (NEGFS - POSFS))'calculate y location
  IF FIRSTSCAN% THEN VOLTY% = volts%
  LINE (S% - 1, VOLTY%)-(S%, volts%), SCOLOR%   'plot it
  VOLTY% = volts%                               'save current y location
IF (SCAN& MOD 10 = 0) OR (SCANINTERVAL# >= .1) THEN
  IF LASTDIGINP% < 256 THEN
    LOCATE 30, 66: PRINT DIGINP$(LASTDIGINP%);
    LASTDIGIN% = 256
  END IF
  LOCATE 29, 7: PRINT USING "  #########  "; SCAN&;
  LOCATE 27, 8: PRINT USING " ##.######   "; volts
  PRINT "Buffer="; LOC(1);
END IF
RETURN

SETUPSCREEN:
  CLS
  S% = 1: TSCREEN = 639 * SCANINTERVAL#      'TSCREEN = time for one screen
  LINE (0, 0)-(0, 400), 2                    'vertical axis
  IF VDIV <= 0 THEN VDIV = 1                 'check bounds for VDIV
  IF NEGFS >= POSFS THEN POSFS = NEGFS + 1   'check bounts for NEGFS and POSFS
  VINDEX& = (399 / (ABS(POSFS - NEGFS) / VDIV))
  IF VINDEX& > 399 THEN VINDEX& = 399        'check bounds on VINDEX&
  IF VINDEX& < 1 THEN VINDEX& = 1
  FOR N% = 0 TO 399 STEP VINDEX&
    LINE (0, N% - 1)-(0, N% + 1), 15         'index vertical axis
  NEXT N%
  LINE (1, 400)-(640, 400), 2                'horizontal axis
  IF HDIV = 0 THEN                           'initialize HINDEX% and HDIV
    GOSUB GETHINDEX
  ELSE
    HINDEX% = 639 / (TSCREEN / HDIV): HSEC = HDIV
  END IF
  IF HINDEX% > 639 THEN HINDEX% = 639         'check bounds on HINDEX%
  IF HINDEX% < 1 THEN HINDEX% = 1
  FOR N% = 0 TO 639 STEP HINDEX%
    FOR NK% = 0 TO 399 STEP VINDEX&
	PSET (N%, NK%), 15                     'index marks on screen
    NEXT NK%
    LINE (N% - 1, 400)-(N% + 1, 400), 15     'index horizontal axis
  NEXT N%
  COLOR 7: LOCATE 27, 1: PRINT "VOLTS:";
  LOCATE 28, 23: COLOR 3: PRINT "P";
  COLOR 7: PRINT USING "lus   FS #######.### mv"; POSFS * 1000;
  LOCATE 28, 50: COLOR 3: PRINT "H";
  COLOR 7: PRINT USING "oriz   #####.### sec/div"; HSEC;
  LOCATE 29, 1: PRINT "SCAN";
  COLOR 3: LOCATE 27, 23: PRINT "C";
  COLOR 7: LOCATE 27, 24: PRINT "hannel"; SINGLECHAN%;
  LOCATE 29, 23: COLOR 3: PRINT "M";
  COLOR 7: PRINT USING "inus  FS #######.### mv"; NEGFS * 1000;
  LOCATE 29, 50: COLOR 3: PRINT "V";
  COLOR 7: PRINT USING "ert  #######.### mv/div"; VDIV * 1000;
  LOCATE 30, 23: COLOR 3: PRINT "E";
  COLOR 7: PRINT "xit";
  LOCATE 30, 33: PRINT "DigO";
  LOCATE 30, 37: COLOR 3: PRINT "U";
  COLOR 7: PRINT "t ";
  IF LASTDIGOUT% < 256 THEN PRINT LASTDIGOUT%;  ELSE PRINT " 0";
  LOCATE 30, 49: PRINT "D"; : COLOR 3: PRINT "A"; : COLOR 7: PRINT "CS";
  LOCATE 30, 60: COLOR 3: PRINT "D"; : COLOR 7: PRINT "igIn";
  LOCATE 30, 1: PRINT USING "Sec/Scrn #####.##"; TSCREEN;
RETURN

CLEARMENU:
  LOCATE 27, 35: PRINT SPACE$(45);
RETURN

GETHINDEX:                                      'calculate inital HINDEX
  HSEC = .1
RETRY: HSEC = HSEC * 10: IF TSCREEN / HSEC > 40 THEN GOTO RETRY
DONEHINDEX:
 HINDEX% = 639 / (TSCREEN / HSEC)
RETURN

'**********************************************************************
'* routine to avoid use of INPUT function, required if EVENT trapping *
'*    is to work smoothly during keyboard input                       *
'**********************************************************************

GETINPUT:                        'Take numeric input until Enter is pressed
KEYINPUT$ = ""
GETNEXTKEY:
  S$ = INPUT$(1)
  IF S$ = CHR$(13) THEN RETURN
  S = ASC(S$)
  IF S = 8 THEN GOSUB BACKSPACE
				'get char between "." and "9" but not "," or "/"
  IF (S >= 43 AND S <= 57) AND S <> 44 AND S <> 47 THEN
    KEYINPUT$ = KEYINPUT$ + S$: PRINT S$;
  END IF
GOTO GETNEXTKEY
						   
BACKSPACE:
  IF LEN(KEYINPUT$) > 0 THEN
    KEYINPUT$ = LEFT$(KEYINPUT$, LEN(KEYINPUT$) - 1)   'one less character
    LOCATE CSRLIN, (POS(0) - 1): PRINT " "; : LOCATE CSRLIN, (POS(0) - 1)
  END IF
RETURN



